defmodule Test12.Test do
  @moduledoc """
  Functions in this module request the execution of a task within
  a poolboy process.  The poolboy process is implemented with a
  GenServer module.
  """

  @timeout 60000

  @doc """
  Calculate `count` means on column in dataset using poolboy processes.
  Each poolboy process holds and uses an erlport/python process.
  """
  @spec start_mean(integer(), String.t(), String.t(), boolean()) :: :ok
  def start_mean(count, data_file_name, column_name, verbose) do
    1..count
    |> Enum.map(fn i -> async_call_mean(i, data_file_name, column_name, verbose) end)
    |> Enum.each(fn task -> await_and_inspect_mean(task) end)
  end

  defp async_call_mean(index, data_file_name, column_name, verbose) do
    Task.async(fn ->
      :poolboy.transaction(
        :worker,
        fn pid ->
          GenServer.call(
            pid,
            {:mean, index, data_file_name, column_name, verbose})
        end,
        @timeout
      )
    end)
  end

  defp await_and_inspect_mean(task) do
    task
    |> Task.await(@timeout)
    |> (fn ({:result, value}) ->
      case value do
        {:not_found, msg} ->
          IO.puts(msg)
        {:invalid_column, msg} ->
          IO.puts(msg)
        {:ok, {index, data_file_name, column_name, mean}} ->
          IO.puts("index: #{index},  df: #{data_file_name}  col: #{column_name}  mean: #{mean}")
      end
    end).()
  end

  @doc """
  Count the number of elements in an XML document.
  Do it `count` times using poolboy processes.
  Each poolboy process holds and uses an erlport/python process.
  """
  @spec start_element_count(integer(), String.t(), boolean()) :: :ok
  def start_element_count(count, xml_file_name, verbose) do
    1..count
    |> Enum.map(fn i -> async_call_element_count(i, xml_file_name, verbose) end)
    |> Enum.each(fn task -> await_and_inspect_element_count(task) end)
  end

  defp async_call_element_count(index, xml_file_name, verbose) do
    Task.async(fn ->
      :poolboy.transaction(
        :worker,
        fn pid ->
          GenServer.call(
            pid,
            {:xml_count, index, xml_file_name, verbose})
        end,
        @timeout
      )
    end)
  end

  defp await_and_inspect_element_count(task) do
    task
    |> Task.await(@timeout)
    |> (fn ({:result, {index, filename, count}}) ->
      IO.puts("index: #{index},  file: #{filename}  element count: #{count}")
    end).()
  end

  def stop(count) do
    Enum.each(1..count, fn (_x) ->
      Task.async(fn ->
        :poolboy.transaction(
          :worker,
          fn pid ->
            GenServer.stop(pid, :killed)
          end,
          @timeout
        )
      end)
    end)
  end

end
